@@ -50,14 +50,16 @@
 #include "rb-dialog.h"
 #include "rb-preferences.h"
 #include "rb-daap-src.h"
+#include "rb-daap-record-factory.h"
+#include "rb-rhythmdb-dmap-db-adapter.h"
 
-#include "rb-daap-connection.h"
-#include "rb-daap-mdns-browser.h"
 #include "rb-daap-dialog.h"
 #include "rb-daap-plugin.h"
 
 #include "rb-static-playlist-source.h"
 
+#include <libdmapsharing/dmap.h>
+
 static void rb_daap_source_dispose (GObject *object);
 static void rb_daap_source_set_property  (GObject *object,
 					  guint prop_id,
@@ -392,7 +394,7 @@ ask_password (RBDAAPSource *source, const char *name, const char *keyring)
 }
 
 static char *
-connection_auth_cb (RBDAAPConnection *connection,
+connection_auth_cb (DMAPConnection   *connection,
 		    const char       *name,
 		    RBDAAPSource     *source)
 {
@@ -440,8 +442,8 @@ connection_auth_cb (RBDAAPConnection *connection,
 }
 
 static void
-connection_connecting_cb (RBDAAPConnection     *connection,
-			  RBDAAPConnectionState state,
+connection_connecting_cb (DMAPConnection       *connection,
+			  DMAPConnectionState   state,
 			  float		        progress,
 			  RBDAAPSource         *source)
 {
@@ -452,20 +454,20 @@ connection_connecting_cb (RBDAAPConnection     *connection,
 	rb_debug ("DAAP connection status: %d/%f", state, progress);
 
 	switch (state) {
-	case DAAP_GET_INFO:
-	case DAAP_GET_PASSWORD:
-	case DAAP_LOGIN:
+	case DMAP_GET_INFO:
+	case DMAP_GET_PASSWORD:
+	case DMAP_LOGIN:
 		source->priv->connection_status = _("Connecting to music share");
 		break;
-	case DAAP_GET_REVISION_NUMBER:
-	case DAAP_GET_DB_INFO:
-	case DAAP_GET_SONGS:
-	case DAAP_GET_PLAYLISTS:
-	case DAAP_GET_PLAYLIST_ENTRIES:
+	case DMAP_GET_REVISION_NUMBER:
+	case DMAP_GET_DB_INFO:
+	case DMAP_GET_SONGS:
+	case DMAP_GET_PLAYLISTS:
+	case DMAP_GET_PLAYLIST_ENTRIES:
 		source->priv->connection_status = _("Retrieving songs from music share");
 		break;
-	case DAAP_LOGOUT:
-	case DAAP_DONE:
+	case DMAP_LOGOUT:
+	case DMAP_DONE:
 		source->priv->connection_status = NULL;
 		break;
 	}
@@ -474,7 +476,7 @@ connection_connecting_cb (RBDAAPConnection     *connection,
 
 	rb_source_notify_status_changed (RB_SOURCE (source));
 
-	is_connected = rb_daap_connection_is_connected (connection);
+	is_connected = dmap_connection_is_connected (DMAP_CONNECTION (connection));
 
 	g_object_get (source, "plugin", &plugin, NULL);
 	g_assert (plugin != NULL);
@@ -491,7 +493,7 @@ connection_connecting_cb (RBDAAPConnection     *connection,
 }
 
 static void
-connection_disconnected_cb (RBDAAPConnection *connection,
+connection_disconnected_cb (DMAPConnection   *connection,
 			    RBDAAPSource     *source)
 {
 	GdkPixbuf *icon;
@@ -527,13 +529,13 @@ release_connection (RBDAAPSource *daap_source)
 }
 
 static void
-_add_location_to_playlist (RBRefString *uri, RBStaticPlaylistSource *source)
+_add_location_to_playlist (const char *uri, RBStaticPlaylistSource *source)
 {
-	rb_static_playlist_source_add_location (source, rb_refstring_get (uri), -1);
+	rb_static_playlist_source_add_location (source, uri, -1);
 }
 
 static void
-rb_daap_source_connection_cb (RBDAAPConnection *connection,
+rb_daap_source_connection_cb (DMAPConnection   *connection,
 			      gboolean          result,
 			      const char       *reason,
 			      RBSource         *source)
@@ -564,9 +566,9 @@ rb_daap_source_connection_cb (RBDAAPConnection *connection,
 		      "shell", &shell,
 		      "entry-type", &entry_type,
 		      NULL);
-	playlists = rb_daap_connection_get_playlists (RB_DAAP_CONNECTION (daap_source->priv->connection));
+	playlists = dmap_connection_get_playlists (DMAP_CONNECTION (daap_source->priv->connection));
 	for (l = playlists; l != NULL; l = g_slist_next (l)) {
-		RBDAAPPlaylist *playlist = l->data;
+		DMAPPlaylist *playlist = l->data;
 		RBSource *playlist_source;
 		char *sorting_name;
 
@@ -591,7 +593,9 @@ rb_daap_source_activate (RBSource *source)
 {
 	RBDAAPSource *daap_source = RB_DAAP_SOURCE (source);
 	RBShell *shell = NULL;
-	RhythmDB *db = NULL;
+	DMAPRecordFactory *factory;
+	RhythmDB *rdb = NULL;
+	DMAPDb *db = NULL;
 	char *name = NULL;
 	RhythmDBEntryType type;
 
@@ -601,17 +605,20 @@ rb_daap_source_activate (RBSource *source)
 
 	g_object_get (daap_source,
 		      "shell", &shell,
-		      "entry-type", &type,
 		      "name", &name,
+		      "entry-type", &type,
 		      NULL);
-	g_object_get (shell, "db", &db, NULL);
+	g_object_get (shell, "db", &rdb, NULL);
+	db = DMAP_DB (rb_rhythmdb_dmap_db_adapter_new (rdb, type));
 
-	daap_source->priv->connection = rb_daap_connection_new (name,
+	factory = DMAP_RECORD_FACTORY (rb_daap_record_factory_new ());
+
+	daap_source->priv->connection = dmap_connection_new (name,
 								daap_source->priv->host,
 								daap_source->priv->port,
 								daap_source->priv->password_protected,
 								db,
-								type);
+								factory);
         g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, type);
 	g_object_add_weak_pointer (G_OBJECT (daap_source->priv->connection), (gpointer *)&daap_source->priv->connection);
 
@@ -630,16 +637,16 @@ rb_daap_source_activate (RBSource *source)
                           G_CALLBACK (connection_disconnected_cb),
 			  source);
 
-	rb_daap_connection_connect (RB_DAAP_CONNECTION (daap_source->priv->connection),
-				    (RBDAAPConnectionCallback) rb_daap_source_connection_cb,
+	dmap_connection_connect (DMAP_CONNECTION (daap_source->priv->connection),
+				    (DMAPConnectionCallback) rb_daap_source_connection_cb,
 				    source);
 
-	g_object_unref (G_OBJECT (db));
+	g_object_unref (G_OBJECT (rdb));
 	g_object_unref (G_OBJECT (shell));
 }
 
 static void
-rb_daap_source_disconnect_cb (RBDAAPConnection *connection,
+rb_daap_source_disconnect_cb (DMAPConnection   *connection,
 			      gboolean          result,
 			      const char       *reason,
 			      RBSource         *source)
@@ -704,8 +711,8 @@ rb_daap_source_disconnect (RBDAAPSource *daap_source)
 
 	/* keep the source alive until the disconnect completes */
 	g_object_ref (daap_source);
-	rb_daap_connection_disconnect (daap_source->priv->connection,
-				       (RBDAAPConnectionCallback) rb_daap_source_disconnect_cb,
+	dmap_connection_disconnect (daap_source->priv->connection,
+				       (DMAPConnectionCallback) rb_daap_source_disconnect_cb,
 				       daap_source);
 
 	/* wait until disconnected */
@@ -726,7 +733,7 @@ rb_daap_source_show_popup (RBSource *source)
 	return TRUE;
 }
 
-GstStructure *
+SoupMessageHeaders *
 rb_daap_source_get_headers (RBDAAPSource *source,
 			    const char *uri)
 {
@@ -735,7 +742,7 @@ rb_daap_source_get_headers (RBDAAPSource *source,
 		return NULL;
 	}
 
-	return rb_daap_connection_get_headers (source->priv->connection, uri);
+	return dmap_connection_get_headers (source->priv->connection, uri);
 }
 
 static char *
@@ -758,16 +765,6 @@ rb_daap_source_get_status (RBSource *source,
 {
 	RBDAAPSource *daap_source = RB_DAAP_SOURCE (source);
 
-	if (text != NULL) {
-		*text = NULL;
-	}
-	if (progress_text != NULL) {
-		*progress_text = NULL;
-	}
-	if (progress != NULL) {
-		*progress = 0.0;
-	}
-
 	if (daap_source->priv->connection_status != NULL) {
 		if (text != NULL) {
 			*text = g_strdup (daap_source->priv->connection_status);
@@ -795,4 +792,3 @@ rb_daap_source_get_playback_uri (RhythmDBEntry *entry, gpointer data)
 
 	return g_strdup (location);
 }
-
