@@ -106,12 +106,14 @@ struct RBShellPreferencesPrivate
 	GtkWidget *artist_check;
 	GtkWidget *album_check;
 	GtkWidget *genre_check;
+	GtkWidget *comment_check;
 	GtkWidget *duration_check;
 	GtkWidget *track_check;
 	GtkWidget *rating_check;
 	GtkWidget *play_count_check;
 	GtkWidget *last_played_check;
 	GtkWidget *first_seen_check;
+	GtkWidget *bpm_check;
 	GtkWidget *quality_check;
 	GtkWidget *year_check;
 	GtkWidget *location_check;
@@ -216,6 +218,8 @@ rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 		GTK_WIDGET (gtk_builder_get_object (builder, "album_check"));
 	shell_preferences->priv->genre_check =
 		GTK_WIDGET (gtk_builder_get_object (builder, "genre_check"));
+	shell_preferences->priv->comment_check =
+		GTK_WIDGET (gtk_builder_get_object (builder, "comment_check"));
 	shell_preferences->priv->duration_check =
 		GTK_WIDGET (gtk_builder_get_object (builder, "duration_check"));
 	shell_preferences->priv->track_check =
@@ -228,6 +232,8 @@ rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 		GTK_WIDGET (gtk_builder_get_object (builder, "last_played_check"));
 	shell_preferences->priv->quality_check =
 		GTK_WIDGET (gtk_builder_get_object (builder, "quality_check"));
+	shell_preferences->priv->bpm_check =
+		GTK_WIDGET (gtk_builder_get_object (builder, "bpm_check"));
 	shell_preferences->priv->year_check =
 		GTK_WIDGET (gtk_builder_get_object (builder, "year_check"));
 	shell_preferences->priv->first_seen_check =
@@ -450,6 +456,8 @@ rb_shell_preferences_column_check_changed_cb (GtkCheckButton *butt,
 		colname = "RHYTHMDB_PROP_ALBUM";
 	else if (butt == GTK_CHECK_BUTTON (shell_preferences->priv->genre_check))
 		colname = "RHYTHMDB_PROP_GENRE";
+	else if (butt == GTK_CHECK_BUTTON (shell_preferences->priv->comment_check))
+		colname = "RHYTHMDB_PROP_COMMENT";
 	else if (butt == GTK_CHECK_BUTTON (shell_preferences->priv->duration_check))
 		colname = "RHYTHMDB_PROP_DURATION";
 	else if (butt == GTK_CHECK_BUTTON (shell_preferences->priv->track_check))
@@ -462,6 +470,8 @@ rb_shell_preferences_column_check_changed_cb (GtkCheckButton *butt,
 		colname = "RHYTHMDB_PROP_LAST_PLAYED";
 	else if (butt == GTK_CHECK_BUTTON (shell_preferences->priv->year_check))
 		colname = "RHYTHMDB_PROP_DATE";
+	else if (butt == GTK_CHECK_BUTTON (shell_preferences->priv->bpm_check))
+		colname = "RHYTHMDB_PROP_BPM";
 	else if (butt == GTK_CHECK_BUTTON (shell_preferences->priv->quality_check))
 		colname = "RHYTHMDB_PROP_BITRATE";
 	else if (butt == GTK_CHECK_BUTTON (shell_preferences->priv->first_seen_check))
@@ -536,6 +546,9 @@ rb_shell_preferences_sync (RBShellPreferences *shell_preferences)
 		rb_shell_preferences_sync_column_button (shell_preferences,
 			       				 shell_preferences->priv->genre_check,
 							 columns, "RHYTHMDB_PROP_GENRE");
+		rb_shell_preferences_sync_column_button (shell_preferences,
+							 shell_preferences->priv->comment_check,
+							 columns, "RHYTHMDB_PROP_COMMENT");
 		rb_shell_preferences_sync_column_button (shell_preferences,
 			       				 shell_preferences->priv->duration_check,
 							 columns, "RHYTHMDB_PROP_DURATION");
@@ -558,8 +571,11 @@ rb_shell_preferences_sync (RBShellPreferences *shell_preferences)
 			       				 shell_preferences->priv->first_seen_check,
 							 columns, "RHYTHMDB_PROP_FIRST_SEEN");
 		rb_shell_preferences_sync_column_button (shell_preferences,
-			       				 shell_preferences->priv->quality_check,
+							 shell_preferences->priv->quality_check,
 							 columns, "RHYTHMDB_PROP_BITRATE");
+		rb_shell_preferences_sync_column_button (shell_preferences,
+							 shell_preferences->priv->bpm_check,
+							 columns, "RHYTHMDB_PROP_BPM");
 		rb_shell_preferences_sync_column_button (shell_preferences,
 			       				 shell_preferences->priv->location_check,
 							 columns, "RHYTHMDB_PROP_LOCATION");
